/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.wsas;

import org.apache.axis2.context.ConfigurationContext;
import org.apache.log4j.Logger;
import org.wso2.utils.FileManipulator;
import org.wso2.utils.ServerConfiguration;
import org.wso2.utils.ServerConfigurationException;
import org.wso2.utils.ServerException;
import org.wso2.wsas.util.Monitor;
import org.wso2.wsas.util.ServerController;
import org.wso2.wsas.util.Utils;
import org.wso2.wsas.util.XmlConfiguration;

import java.io.File;
import java.io.IOException;
import java.util.Map;

/**
 * The main class of the WSO2 WSAS standalone runtime.
 */
public class Main {

    // Server lifecycle commands
    public static final String COMMAND_START = "START";
    public static final String COMMAND_RESTART = "RESTART";
    public static final String COMMAND_STOP = "STOP";
    public static final String COMMAND_RUN = "RUN";
    public static final String COMMAND_STATUS = "STATUS";
    public static final String COMMAND_HELP = "HELP";

    public static Thread shutdownHookThread;

    private WebServer webServer;
    private static ServerConfiguration serverConfig;
    private static Logger log = Logger.getLogger(Main.class);

    public static XmlConfiguration xmlConfiguration;

    public Main() {
        if (System.getProperty(ServerConstants.WSO2WSAS_HOME) == null) {
            System.setProperty(ServerConstants.WSO2WSAS_HOME, ".");
            System.setProperty(ServerConstants.AXIS2_HOME, ".");
        } else {
            System.setProperty(ServerConstants.AXIS2_HOME,
                               System.getProperty(ServerConstants.WSO2WSAS_HOME));
        }
        String serverXMLLocation = Utils.getWsasServerXml();
        try {
            serverConfig = ServerConfiguration.getInstance();
            serverConfig.init(serverXMLLocation);
        } catch (ServerConfigurationException e) {
            throw new RuntimeException(e);
        }
        System.setProperty(ServerConstants.DERBY_HOME,
                           serverConfig.getFirstProperty("Database.Home"));
    }


    public void startServer() {
        startListeners();
    }

    private void startListeners() {
        try {
            webServer = new TomcatServer();
            webServer.start();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public void stop() throws IOException, ServerException {
        ServerManager.getInstance().stop();
        stopListeners();
    }

    public void status() throws IOException {
        ServerManager.getInstance().status();
    }

    public void shutdown() throws ServerException {
        stopListeners();
        cleanupSystem();
    }

    public static void main(String[] args) {
        System.setProperty(ServerConstants.WSO2WSAS_START_TIME,
                           String.valueOf(System.currentTimeMillis()));
        String cmd;
        if (args.length >= 1) {
            cmd = args[0];
        } else {
            printUsages();
            return;
        }

        final Main main = new Main();
        String serverName = serverConfig.getFirstProperty("Name");

        if (cmd.toUpperCase().endsWith(COMMAND_START) || cmd.toUpperCase().endsWith(COMMAND_RUN)) {
            main.startServer();
        } else if (cmd.toUpperCase().endsWith(COMMAND_RESTART)) {
            try {
                ServerController serverController = new ServerController();
                int serverPort =
                        Integer.parseInt(serverConfig.
                                getFirstProperty(ServerConfiguration.COMMAND_LISTENER_PORT));
                serverController.setPort(serverPort);
                serverController.restartServer();
            } catch (Exception e) {
                String msg = "Cannot restart " + serverName;
                log.error(msg, e);
            }
        } else if (cmd.toUpperCase().endsWith(COMMAND_STOP)) {
            try {
                ServerController serverController = new ServerController();
                int serverPort =
                        Integer.parseInt(serverConfig.
                                getFirstProperty(ServerConfiguration.COMMAND_LISTENER_PORT));
                serverController.setPort(serverPort);
                serverController.shutDownServer();
            } catch (Exception e) {
                String msg = "Cannot stop " + serverName;
                log.error(msg, e);
            }
        } else if (cmd.toUpperCase().endsWith(COMMAND_STATUS)) {
            try {
                main.status();
            } catch (IOException e) {
                String msg = "Cannot get status of " + serverName;
                log.error(msg, e);
            }
        } else if (cmd.toUpperCase().endsWith(COMMAND_HELP)) {
            printUsages();
        } else {
            printUsages();
        }
    }

    private static void printUsages() {
        System.out.println("======================================================");
        System.out.println("=                WSO2 WSAS Help                      =");
        System.out.println("======================================================");
        System.out.println();
        System.out.println(" Usage : WSO2WSAS <action>");
        System.out.println();
        System.out.println("<action> \t\t : Current actions supported - " +
                           COMMAND_START + ", " +
                           COMMAND_RUN + ", " +
                           COMMAND_RESTART + ", " +
                           COMMAND_STATUS + ", " +
                           COMMAND_STOP + ", " +
                           COMMAND_HELP);
        System.out.println();
    }

    public ConfigurationContext getConfigContext() {
        return ServerManager.getInstance().configContext;
    }

    public void setMonitor(Monitor m) {
        ServerManager.getInstance().monitor = m;
    }

    public void stopListeners() throws ServerException {
        if (webServer != null) {
            try {
                webServer.stop();
            } catch (Exception axisFault) {
                throw new ServerException(axisFault);
            }
        }
        webServer = null;
    }

    private void cleanupSystem() {
        log.info("Cleaning up system...");
        new FileManipulator().deleteDir(new File(serverConfig.getFirstProperty("WorkDirectory")));
        ConfigurationContext configContext = ServerManager.getInstance().configContext;
        if (configContext != null) {
            Object property = configContext.getProperty(ServerConstants.FILE_RESOURCE_MAP);
            if (property != null) {
                ((Map) property).clear();
            }
        }
    }
}
